/*
 * screen_encrypt.c: FBCrypt-HVM screen encrypt
 *
 */

#include <xen/lib.h>
#include <asm/page.h>       // virt_to_mfn
#include <xen/domain_page.h>// map_domain_page
#include <xen/xmalloc.h>    // xmalloc_bytes
#include <xen/string.h>		// strlen
#include <asm/system.h>     // hypercall
#include <asm/current.h>    // current
#include <xen/mm.h>         // alloc_domheap_pages
#include <xen/sched.h>      // struct domain

#include <crypto/fbcrypt/hvm/hvm.h>
#include <crypto/fbcrypt/cyassl/cyassl.h>
#include "../cyassl/RC5/RC5_48.h"

/* resolution */
#if FBCRYPT_XGA
	#define WIDTH 1024
	#define HEIGHT 768
#elif FBCRYPT_VGA
	#define WIDTH 800
	#define HEIGHT 600
#endif

#define PIXEL 3 /* byte per pixel */

#define VRAM_PAGES 1024

/* HVM_VRAM INFO -------------------- */
typedef struct fbcrypt_hvm_vram {
	int domid;
	uint8_t *va_gvram[VRAM_PAGES];
	uint8_t *va_dvram[VRAM_PAGES];
	uint32_t vga_ram_size;
	unsigned long mfn_dvram[VRAM_PAGES];
	unsigned long mfn_gvram[VRAM_PAGES];
	unsigned long nr_pages;
	int cnt;
    rc5_ctx rc5[1200]; /* fix */
} FBCRYPT_HVM_VRAM;

FBCRYPT_HVM_VRAM info;
unsigned char *HVMSCREEN_KEY = (unsigned char *)"08237061";


static void set_t_vram_info(int domid, unsigned long mfn, int index)
{
	info.mfn_gvram[index] = mfn;
	info.va_gvram[index] = mfn_to_virt(mfn);

	#if DEBUG_HVM_SC
	if ( index==0 || index%100==0 || index==1023 ) {
		printk("GET_GUEST_VRAM: index:%d mfn:%ld va:%lx value:%d\n",
				index,mfn,(unsigned long)info.va_gvram[index],*info.va_gvram[index]);
	}
	#endif
}


static void create_dummy_vram(int domid)
{
    struct domain *guest;
    struct page_info *dummy_pg;
	int i;

 	/* Get guest domain */
	guest = rcu_lock_domain_by_id(domid);
	/* create dummy_vram */
	/* 10: 2^10 = 1024 pages */
	dummy_pg = alloc_domheap_pages(guest,10,1);
	info.va_dvram[0] = __page_to_virt(dummy_pg);
	/* Get dummy VRAM mfn */
	info.mfn_dvram[0] = __virt_to_mfn(info.va_dvram[0]);

	for ( i=1; i<VRAM_PAGES; i++ ) {
		info.mfn_dvram[i] = info.mfn_dvram[0]+i;
		info.va_dvram[i] = mfn_to_virt(info.mfn_dvram[i]);
	}

	printk("CREATE_DUMMY_VRAM: domid:%d mfn:%ld va:%lx value:%d\n",
			domid,info.mfn_dvram[0],(unsigned long)info.va_dvram[0],*info.va_dvram[0]);
}


static void debug_print(int cnt, unsigned long gpfn, int flag)
{
	if ( cnt==0 || cnt%100==0  || cnt==1023 ) {
		printk("REPLACE_DUMMY_VRAM:domid:%d t_mfn[%d]:%ld --> d_mfn[%d]:%ld gpfn:%ld d_flag:%d\n",
				info.domid,cnt,info.mfn_gvram[cnt],cnt,info.mfn_dvram[cnt],gpfn,flag);
	}
}


static void is_create_dummy_vram(int domid, unsigned gpfn, int *flag)
{
	if ( gpfn == 1044480 && *flag == 0 ) { 
		*flag = 1;
		info.domid = domid;
		info.cnt = 0;
		create_dummy_vram(domid);
	}
}


static void prepare_rc5(int domid)
{
	int rounds = 16;
	int i;
	
	/* init rc5 */
    for( i=0; i < 1200; i++ ){ /* fix */
        rc5_init(&(info.rc5[i]),rounds);
        rc5_key(&(info.rc5[i]), HVMSCREEN_KEY,strlen((char *)HVMSCREEN_KEY),i);
    }

	printk("SET RC5: key:%s\n",HVMSCREEN_KEY);
}


/* arch/x86/mm.c @ mod_l1_entry */
unsigned long replace_dummy_vram(int domid, unsigned long mfn, unsigned long gpfn)
{
	unsigned long mfn_dvram = 0;
	static int dummy_create_flag = 0;

	/* create dummy_vram when guest booted only */
	is_create_dummy_vram(domid,gpfn,&dummy_create_flag);

	/* replace mfn with dummy_vram_mfn */
	if ( gpfn >= 1044480 && gpfn <= 1045503 ) {
		mfn_dvram = info.mfn_dvram[0] + info.cnt;
		info.mfn_dvram[info.cnt] = mfn_dvram;
		set_t_vram_info(domid,mfn,info.cnt);
		debug_print(info.cnt,gpfn,dummy_create_flag);
		info.cnt++;
		/* そのドメインのVRAM全てをすり替えたら次回起動のためにフラグoff */
		if ( info.cnt==VRAM_PAGES ) { 
			dummy_create_flag = 0;
			prepare_rc5(domid);
		}
		return mfn_dvram;
	}

	/* 連続してVRAMのPTE変更がくるとは限らない */
	return mfn;
}


static void set_dirty_bit(int idx, uint8_t dirty_bit[], unsigned long bitmap, int *sync_pages, int nr)
{
	int i;
	int tile = 64;
	static int cnt = 0;
	for ( i=0; i<tile; i++ ) {
		dirty_bit[cnt] = (bitmap >> i) & 0x0000000000000001;
		if ( dirty_bit[cnt] == 1 ) (*sync_pages)++;
		cnt++;
	}
	if ( nr == 576 && cnt == 576 ) cnt = 0; /* XGA 1024x768 */
	if ( nr == 450 && cnt == 448 ) cnt = 0; /* VGA 800x600 */
}


static void stuff_pixel(int cnt, uint32_t pixels[], uint8_t red, uint8_t green, uint8_t blue)
{
	/* pixels[n] = R|G|B */
	pixels[cnt] = red;
	pixels[cnt] <<= 8;
	pixels[cnt] |= green;
	pixels[cnt] <<= 8;
	pixels[cnt] |= blue;
}


/* pixel across pages */
static void page2pixels(int idx, uint8_t *gvram[], uint32_t pixels[])
{
	int cnt = 0;	/* pixel_index */
	int offset=0;	/* offset in page */
	uint8_t red;
	uint8_t green;
	uint8_t blue;
	uint8_t *current_page; /* B|G|R */
	uint8_t *next_page;
	uint8_t *prev_page;

	current_page = gvram[idx];

	switch ( idx % 3 ) {
	case 0: 
		for ( cnt=0; cnt<1366; cnt++ ) {
			if ( cnt == 1365 ) {
		 		blue = *(current_page + offset);
				/* ----------------------- next_page +2byte */
				offset = 0;
				next_page = gvram[idx+1];
		 		green = *(next_page + offset++);
		 		red = *(next_page + offset);
				stuff_pixel(cnt,pixels,red,green,blue);
			} 
			else {
		 		blue = *(current_page + offset++);
		 		green= *(current_page + offset++);
		 		red = *(current_page + offset++);
				stuff_pixel(cnt,pixels,red,green,blue);
			}
		}
	 	break;
	 
	case 1: 
		offset += 2; /* skip 2byte of top */
		for ( cnt=0; cnt<1364; cnt++ ) {
			blue = *(current_page + offset++);
			green = *(current_page + offset++);
			red = *(current_page + offset++);
			stuff_pixel(cnt,pixels,red,green,blue);
		}
		break;

	case 2:
		for ( cnt=0; cnt<1366; cnt++ ) {
			if ( cnt == 0 ) {
				/* ----------------------- prev_page +2byte */
				prev_page = gvram[idx-1];
				offset += PAGE_SIZE-2;
		 		blue = *(prev_page + offset++);
		 		green= *(prev_page + offset);

				/* ----------------------- current_page */
				offset = 0;
		 		red = *(current_page + offset++);
				stuff_pixel(cnt,pixels,red,green,blue);
			} 
			else {
				blue = *(current_page + offset++);
				green = *(current_page + offset++);
				red = *(current_page + offset++);
				stuff_pixel(cnt,pixels,red,green,blue);
			}
		}
		break;
	}
}


static void pixels2page(int idx, uint32_t pixels[], uint8_t *dvram[]) 
{
	int cnt = 0;	/* pixel_index */
	int offset=0;	/* offset in page */
	uint8_t *current_page;
	uint8_t *next_page;
	uint8_t *prev_page;

	current_page = dvram[idx];

	switch ( idx % 3 ) {
	case 0: 
		for ( cnt=0; cnt<1366; cnt++ ) {
			if ( cnt == 1365 ) {
				next_page = dvram[idx+1];
				*(current_page +offset) = pixels[cnt] & 0x000000ff;	/* blue */
				*(next_page+0) = (pixels[cnt] >> 8) & 0x000000ff; 	/* green */
				*(next_page+1) = (pixels[cnt] >> 16) & 0x000000ff; 	/* red */
			}
			else {
				*(current_page + offset++) = pixels[cnt] & 0x000000ff; 			/* blue */
				*(current_page + offset++) = (pixels[cnt] >> 8) & 0x000000ff; 	/* green */
				*(current_page + offset++) = (pixels[cnt] >> 16) & 0x000000ff; 	/* red */
			}
		}
		break;

	case 1:
		offset += 2; /* skip 2byte of top */
		for ( cnt=0; cnt<1364; cnt++ ) {
			*(current_page + offset++) = pixels[cnt] & 0x000000ff; 			/* blue */
			*(current_page + offset++) = (pixels[cnt] >> 8) & 0x000000ff; 	/* green */
			*(current_page + offset++) = (pixels[cnt] >> 16) & 0x000000ff; 	/* red */
		}
		break;

	case 2:
		for ( cnt=0; cnt<1366; cnt++ ) {
			if ( cnt == 0 ) {
				prev_page = dvram[idx-1];
				*(prev_page+PAGE_SIZE-2) = pixels[cnt] & 0x000000ff; 		/* blue */
				*(prev_page+PAGE_SIZE-1) = (pixels[cnt] >> 8) & 0x000000ff; /* green */
				*current_page = (pixels[cnt] >> 16) & 0x000000ff; 			/* red */
				offset++;
			}
			else {
				*(current_page + offset++) = pixels[cnt] & 0x000000ff; 			/* blue */
				*(current_page + offset++) = (pixels[cnt] >> 8) & 0x000000ff; 	/* green */
				*(current_page + offset++) = (pixels[cnt] >> 16) & 0x000000ff; 	/* red */
			}
		}
		break;
	}
}


static void crypt_2pixels(int cnt, int x, int y, uint32_t pixels[])
{
	unsigned long crypt[2];

	crypt[0] = pixels[cnt] & 0x0000000000ffffff; 	/* (0,0) */
	crypt[1] = pixels[cnt+1] & 0x0000000000ffffff; /* (1,0) */

	rc5_encrypt(&(info.rc5[y]),crypt,1,x);

	pixels[cnt] = crypt[0];
	pixels[cnt+1] = crypt[1];
}


static void encrypt_pixels(int idx, uint32_t pixels[])
{
	int x = 0;
	int y = 0;
	int cnt = 0; 
	y = idx+(idx/3);

	switch ( idx % 3 ) {

	case 0: /* 1366 pixels */
		for ( x=0; x < WIDTH; x+=2 ) {
			crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
		}
		y++;
		for ( x=0; x < 342; x+=2 ) {
			crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
		}
		break;

	case 1: /* 1364 pixels */
		for ( x=342; x < WIDTH; x+=2 ) {
			crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
		}
		y++;
		for ( x=0; x < 682; x+=2 ) {
			crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
		}
		break;

	case 2: /* 1366 pixels */
		for ( x=682; x < WIDTH; x+=2 ) {
			crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
		}
		y++;
		for ( x=0; x < WIDTH; x+=2 ) {
			crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
		}
		break;
	}
}


#if FBCRYPT_VGA
static void encrypt_pixels_v2(int idx, uint32_t pixels[])
{
	int x = 0;
	int y = 0;
	int cnt = 0; 
	unsigned long a = 0;
	unsigned long b = 0;
	int loop = 0;

	a = idx * 4096;
	b = a/3;
	x = b % WIDTH;
	y = b / WIDTH;

	//printk("a:%ld b:%ld x:%d y:%d idx:%d\n",a,b,x,y,idx);

	switch ( idx % 3 ) {

	case 0: /* 1366 pixels */
		for ( loop=0; loop<1366; loop++ ) {
			//crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
			if ( x == WIDTH ) { x=0; y++; } 
			else x+=2;
		}
		break;

	case 1: /* 1364 pixels */
		for ( loop=0; loop<1364; loop++ ) {
			//crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
			if ( x == WIDTH ) { x=0; y++; }
			else x+=2;
		}
		break;

	case 2: /* 1366 pixels */
		for ( loop=0; loop<1366; loop++ ) {
			//crypt_2pixels(cnt,x,y,pixels);
			cnt += 2;
			if ( x == WIDTH ) { x=0; y++; }
			else x+=2;
		}
		break;
	}
}
#endif

/* common/common.c @ shadow_track_dirty_vram */
void sync_vram(int domid, unsigned long *bitmap, 
		unsigned long dirty_size, unsigned long nrpages)
{
	/* --------------- XGA 1024x768  	* ---------------- VGA 800x600
	 * dirty_size: 72 					* dirty_size: 57
	 * sizeof(bitmap): 8				* sizeof(bitmap): 8
	 * nrpages: 576						* nrpages: 450 --> 448
	 */

	int i;
	int bitmap_row = dirty_size / sizeof(bitmap);
	int sync_pages = 0;
	/* due to reduce allocating cost, adds static */
	static uint8_t dirty_bit[576] = {0};
	static uint32_t pixels[1366]; /* MAX: 1366pixels/PAGE */

	#if TIME_HVM_SCREEN
    s_time_t start;
	s_time_t end;
	/* don't use double. VMM can't deal it */
	s_time_t overhead;
	static unsigned long logid = 0;
	#endif

	#if TIME_HVM_SCREEN
	start = NOW();
	#endif
	
	/* get updated page from bitmap */
	#if DEBUG_HVM_SC
	printk("bitmap ==================================== \n");
	#endif
	for ( i=0; i < bitmap_row; i++ ) {
		#if DEBUG_HVM_SC
		printk("bitmap[%d]: %08lx\n",i,bitmap[i]);
		#endif
		set_dirty_bit(i,dirty_bit,bitmap[i],&sync_pages,nrpages);
	}

	#if DEBUG_HVM_SC
	printk("bitmap_row:%d nrpages:%ld\n",bitmap_row,nrpages);
	#endif

	/* processing each a page */
	for ( i=0; i<nrpages; i++ ) {
		if ( dirty_bit[i] == 1 ) {
			#if FBCRYPT_XGA
			page2pixels(i,info.va_gvram,pixels);
			encrypt_pixels(i,pixels);
			pixels2page(i,pixels,info.va_dvram);

			#elif FBCRYPT_VGA
			page2pixels(i,info.va_gvram,pixels);
			encrypt_pixels_v2(i,pixels);
			pixels2page(i,pixels,info.va_dvram);
			#endif

			/* clear dirty */
			dirty_bit[i] = 0;
		}
		/* sync all pages */
		//memcpy(info.va_dvram[i],info.va_gvram[i],PAGE_SIZE);
	}

	#if TIME_HVM_SCREEN
	end = NOW();
	overhead = (end-start)/1000; /* ns -> us */
    printk("id:%ld HVM_VRAM_ENCRYPT:%ld(us) sync_pages:%d\n",logid++,overhead,sync_pages);
	#endif
}

