/*
 * QEMU keysym to keycode conversion using rdesktop keymaps
 *
 * Copyright (c) 2011 Johannes Schindelin
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include <xen/types.h>
#include <asm/system.h>
#include <crypto/fbcrypt/keycode/keycode.h>

static int keysym2scancode(void *kbd_layout, int keysym)
{
    kbd_layout_t *k = kbd_layout;
    if (keysym < MAX_NORMAL_KEYCODE) {
	if (k->keysym2keycode[keysym] == 0) 
	    printk("Warning: no scancode found for keysym %d\n",keysym);
	return k->keysym2keycode[keysym];
    } else {
	int i;
#ifdef XK_ISO_Left_Tab
	if (keysym == XK_ISO_Left_Tab)
	    keysym = XK_Tab;
#endif
	for (i = 0; i < k->extra_count; i++)
	    if (k->keysym2keycode_extra[i].keysym == keysym)
		return k->keysym2keycode_extra[i].keycode;
    }
    return 0;
}

static inline int keysym_is_altgr(void *kbd_layout, int keysym)
{
    kbd_layout_t *k = kbd_layout;
    struct key_range *kr;

    for (kr = k->altgr_range; kr; kr = kr->next)
        if (keysym >= kr->start && keysym <= kr->end){
            return 1;
	}
    return 0;
}

static inline int keysym_is_shift(void *kbd_layout, int keysym)
{
    kbd_layout_t *k = kbd_layout;
    struct key_range *kr;

    for (kr = k->shift_range; kr; kr = kr->next)
        if (keysym >= kr->start && keysym <= kr->end)
            return 1;
    return 0;
}

/* sym => keycode PUBLIC function */
int do_key_event(VncState *vs, int down, uint32_t sym, int mode)
{
    int keycode;
    int shift_keys = 0;
    int shift = 0;
    int altgr = 0;
    int altgr_keys = 0;

    if (sym >= 'A' && sym <= 'Z') { sym = sym - 'A' + 'a'; shift = 1; }
    else { shift = keysym_is_shift(vs->kbd_layout, sym & 0xFFFF); }

    altgr = keysym_is_altgr(vs->kbd_layout, sym & 0xFFFF);

    shift_keys = vs->modifiers_state[0x2a] | vs->modifiers_state[0x36];
    altgr_keys = vs->modifiers_state[0xb8];

    keycode = keysym2scancode(vs->kbd_layout, sym & 0xFFFF);
    if (keycode == 0) {
        printk("Key lost : keysym=0x%x(%d)\n", sym, sym);
        return 0;
    }

	/* HVM ONLY NOT PV -----------------------------*/
	/* keycode is expressed as follow:
	   bit 7    - 0 key pressed, 1 = key released
	   bits 6-0 - translated scancode set 2       
	 */
	if (mode == 1) {
    	if (down) keycode = (keycode & 0x7f);
    	else keycode = (keycode | 0x80);
	}
	/* --------------------------------------------*/
	
	return keycode;
}
