/*-----------------------------------------------
 * xen/fbcrypt/pv/get_store_mfn.c
 *
 * DomainU起動時に仮想CPUレジスタから
   起動情報ページ(start_info)を取得し
   XenStoreリングのmfn(store_mfn)を取得・VMMに登録する
 * ---------------------------------------------*/

#include <xen/config.h>
#include <xen/init.h>
#include <xen/lib.h>
#include <xen/errno.h>
#include <xen/sched.h>
#include <xen/domain.h>
#include <xen/smp.h>
#include <xen/delay.h>
#include <xen/softirq.h>
#include <xen/grant_table.h>
#include <xen/iocap.h>
#include <xen/kernel.h>
#include <xen/multicall.h>
#include <xen/irq.h>
#include <xen/event.h>
#include <xen/console.h>
#include <xen/percpu.h>
#include <xen/compat.h>
#include <xen/acpi.h>
#include <xen/pci.h>
#include <xen/paging.h>
#include <xen/cpu.h>
#include <xen/wait.h>
#include <public/sysctl.h>
#include <asm/regs.h>
#include <asm/mc146818rtc.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/processor.h>
#include <asm/desc.h>
#include <asm/i387.h>
#include <asm/mpspec.h>
#include <asm/ldt.h>
#include <asm/hypercall.h>
#include <asm/hvm/hvm.h>
#include <asm/hvm/support.h>
#include <asm/debugreg.h>
#include <asm/msr.h>
#include <asm/traps.h>
#include <asm/nmi.h>
#include <asm/mce.h>
#include <xen/numa.h>
#include <xen/iommu.h>
#ifdef CONFIG_COMPAT
#include <compat/vcpu.h>
#endif

#include <crypto/fbcrypt/pv/pv.h>


/* FROM Dr.Tadokoro ------------------------------------------EGAWA */
static inline unsigned long
find_target_mfn(struct domain* d, unsigned long target_gfn) {
	struct page_info* page;
	unsigned long ret = INVALID_MFN;
	
	spin_lock(&d->page_alloc_lock);
	page_list_for_each(page, &d->page_list) {
 	 	unsigned long mfn = page_to_mfn(page);
 	 	unsigned long gfn = get_gpfn_from_mfn(mfn);
 	 	if (gfn == target_gfn) {
 	 	       ret = mfn;
 	 	       break;
 		}
  	}
   	spin_unlock(&d->page_alloc_lock);

   	return ret;
}

/* start_info のmfnを取得 */
inline unsigned long
get_start_info2_frame(struct domain* d) {
   	struct vcpu* v = d->vcpu[0];
   	const unsigned long rsi = v->arch.guest_context.user_regs.rsi;
   	const unsigned long gfn = (rsi & ~0xffffffff80000000)>>12;
   	const unsigned long mfn = find_target_mfn(d, gfn);

   	#ifdef DEBUG_STORE_MFN
  	printk("\nDEBUG-INFO: get_start_info2_frame\n");
  	printk("rsi: %lu\n",rsi);
  	printk("gfn: %lu\n",gfn);
  	printk("mfn: %lu\n",mfn);
	#endif

	return mfn;
}

/* arch/x86/domain.c で実行される */
/* 高速な動作のため実行に inline が必須 */
static inline int is_store_mfn2_frame(struct domain* d) {
   	struct start_info* start_info = NULL;
   	unsigned long start_info_mfn = get_start_info2_frame(d);
   	unsigned long store_mfn;

	// VMM の構造体配列の初期化
	// VMM 起動時のみの一回だけ
	static int init_flag = 0;
	if ( init_flag == 0 ) {
		vmm_register_init();
		init_flag = 1;
	}

	if (!mfn_valid(start_info_mfn)) { return 0; }
	start_info = map_domain_page(start_info_mfn);
    if (start_info == NULL) { return 0; }

    store_mfn = start_info->store_mfn;

	// domid == DomainU
	if ( store_mfn != 0 ) {
		vmm_register_info(d->domain_id,store_mfn,VMM_STORE);
	}

   	#ifdef DEBUG_STORE_MFN
	printk("start_info_mfn: %lu\n",start_info_mfn);
	printk("xenstore_mfn: %lu\n",store_mfn);
	#endif

    unmap_domain_page(start_info);

	return 0;
}

